/**************************************************************************
*  Copyright (c) 2015-2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  15.02.2015  mifi  First Version.
**************************************************************************/
#define __IPWEB_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include "tal.h"
#include "ipstack.h"
#include "ipweb.h"

#include "dev/urom.h"

#include "pro/uhttp/mediatypes.h"
#include "pro/uhttp/modules/mod_redir.h"
#include "pro/uhttp/modules/mod_cgi_func.h"
#include "pro/uhttp/streamio.h"

/*=======================================================================*/
/*  Extern                                                               */
/*=======================================================================*/

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/* 
 * Stack definition.
 */
static OS_STACK (task_httpd_stack1, TASK_IP_WEB_STK_SIZE); 
static OS_STACK (task_httpd_stack2, TASK_IP_WEB_STK_SIZE); 

static os_tcb_t TCBHttpd1;
static os_tcb_t TCBHttpd2;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  HttpdTask                                                            */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void HttpdTask (void *p)
{
   (void)p;

   /* Wait for the network */
   while (0 == IPIfaceIsReady())
   {
      OSTimeDly(100);
   }
   
   /* 
    * Wait for a client (browser) and handle its request.
    * This function will only return on unrecoverable errors. 
    */
   StreamClientAccept(HttpdClientHandler, NULL);
   
   TAL_FAILED();
   
   /* Typically this point will be never reached. */
   while(1)
   {
      OSTimeDly(500);
   }
} /* HttpdTask */

/*************************************************************************/
/*  SetupFilesystem                                                      */
/*                                                                       */
/*  Check which filesystem is available.                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SetupFilesystem (void)
{
   /* 
    * Register the default read only filesystem, UROM.
    */
   NutRegisterDevice(&devUrom, 0, 0);
   
   /*
    * UROM is the primary filesystem for the webserver, for the moment.
    */
   HttpRegisterRootPath("UROM:"); 

   /* UROM is the default, register redirector, "Moved Permanently" */
   HttpRegisterRedir("", "/index.htm", 301);

} /* SetupFilesystem */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  IPWebsStart                                                          */
/*                                                                       */
/*  Start the web server.                                                */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void IPWebsStart (void)
{
   /* 
    * Initialize the stream interface. 
    */
   StreamInit();
   
   /* 
    * Register media type defaults. These are configurable
    * in include/cfg/http.h or the Nut/OS Configurator.
    */
   MediaTypeInitDefaults();
   HttpRegisterMediaType("html", "text", "html", MediaTypeHandlerText);
   HttpRegisterMediaType("htm",  "text", "html", MediaTypeHandlerText);
   //HttpRegisterMediaType("cgi", NULL, NULL, HttpCgiFunctionHandler);
   
   /*
    * Setup our filesystem
    */
   SetupFilesystem();
   
   /* Create the HTTP server tasks */
   OSTaskCreate(&TCBHttpd1, HttpdTask, NULL, TASK_IP_WEB_PRIORITY,
                task_httpd_stack1, sizeof(task_httpd_stack1), 
                "HttpdTask1");

   OSTaskCreate(&TCBHttpd2, HttpdTask, NULL, TASK_IP_WEB_PRIORITY,
                task_httpd_stack2, sizeof(task_httpd_stack2), 
                "HttpdTask2");
                    
} /* IPWebsStart */

/*************************************************************************/
/*  time                                                                 */
/*                                                                       */
/*  Return the current Unixtime.                                         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: 0 on success / -1 otherwise                                  */
/*************************************************************************/
time_t time (time_t *t)
{
   time_t Value;
   
   Value = (time_t)OSUnixtimeGet();
   if (t != NULL)
   {
      *t = Value;
   }
   
   return(Value);
} /* time */
/*** EOF ***/
