/**************************************************************************
*  This file is part of the TAL project (Tiny Abstraction Layer)
*
*  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  17.03.2017  mifi  First Version, tested with an Embedded Artists 
*                    LPC4088 Quickstart Board and QSB4088com.
**************************************************************************/
#if defined(USE_BOARD_BF4088)
#define __TALBOARD_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <string.h>
#include "tal.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/* System oscillator rate and RTC oscillator rate */
const uint32_t OscRateIn    = OSCILLATOR_CLOCK_FREQUENCY;
const uint32_t RTCOscRateIn = 32768;

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  tal_BoardEnableCOMx                                                  */
/*                                                                       */
/*  Board and hardware depending functionality to enable the COM port.   */
/*  If this port is not supported by the board, return an error.         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: TAL_OK / error cause                                         */
/*************************************************************************/
TAL_RESULT tal_BoardEnableCOM1 (void)
{
   /* 
    * Use UART0, P0.2 (TX) and P0.3 (RX)  
    */

   /* Turn on UART0 clock */
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_UART0);

   /* Set GPIO P0.2 and P0.3 to UART0 mode */
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 2, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 3, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/

   return(TAL_OK);
} /* tal_BoardEnableCOM1 */

TAL_RESULT tal_BoardEnableCOM2 (void)
{
   return(TAL_ERR_COM_PORT_NOHW);
} /* tal_BoardEnableCOM2 */

TAL_RESULT tal_BoardEnableCOM3 (void)
{
   return(TAL_ERR_COM_PORT_NOHW);
} /* tal_BoardEnableCOM3 */

TAL_RESULT tal_BoardEnableCOM4 (void)
{
   /* 
    * Use UART3, P0.25 (TX) and P0.26 (RX)  
    */

   /* Turn on UART3 clock */
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_UART3);

   /* Set GPIO P0.25 and P0.26 to UART0 mode */
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 25, (IOCON_FUNC3 | IOCON_MODE_INACT | IOCON_DIGMODE_EN));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 26, (IOCON_FUNC3 | IOCON_MODE_INACT | IOCON_DIGMODE_EN | IOCON_FILT_DIS));   /*lint !e845*/

   return(TAL_OK);
} /* tal_BoardEnableCOM4 */

TAL_RESULT tal_BoardEnableCOM5 (void)
{
   /* 
    * Use UART4, P5.4 (TX) and P5.3 (RX)  
    */

   /* Turn on UART4 clock */
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_UART4);

   /* Set GPIO P5.3 and P5.4 to UART4 mode */
   Chip_IOCON_PinMuxSet(LPC_IOCON, 5, 3, (IOCON_FUNC4 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 5, 4, (IOCON_FUNC4 | IOCON_MODE_INACT));   /*lint !e845*/

   return(TAL_OK);
} /* tal_BoardEnableCOM5 */

/*************************************************************************/
/*  tal_BoardEnableCANx                                                  */
/*                                                                       */
/*  Board and hardware depending functionality to enable the CAN port.   */
/*  If this port is not supported by the board, return an error.         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: TAL_OK / error cause                                         */
/*************************************************************************/
TAL_RESULT tal_BoardEnableCAN1 (void)
{
   /* 
    * Use CAN1, P0.0 (RD) and P0.1 (TD)  
    */

   /* Turn on CAN1 and CAN2 clock */
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_CAN1);
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_CAN2);
   
   /* Set GPIO P0.0 and P0.1 to CAN1 mode */
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 0, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 1, (IOCON_FUNC1 | IOCON_MODE_INACT));   /*lint !e845*/

   return(TAL_OK);
} /* tal_BoardEnableCAN1 */

TAL_RESULT tal_BoardEnableCAN2 (void)
{
   /* 
    * Use CAN2, P0.4 (RD) and P0.5 (TD)  
    */

   /* Turn on CAN1 and CAN2 clock */
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_CAN1);
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_CAN2);

   /* Set GPIO P0.4 and P0.5 to CAN2 mode */
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 4, (IOCON_FUNC2 | IOCON_MODE_INACT));   /*lint !e845*/
   Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 5, (IOCON_FUNC2 | IOCON_MODE_INACT));   /*lint !e845*/

   return(TAL_OK);
} /* tal_BoardEnableCAN2 */

/*************************************************************************/
/*  tal_BoardGetMACAddress                                               */
/*                                                                       */
/*  Retrieve the MAC address of the board.                               */
/*  In case of an error, a default address will be used.                 */
/*                                                                       */
/*  In    : pAddress                                                     */
/*  Out   : pAddress                                                     */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
TAL_RESULT tal_BoardGetMACAddress (uint8_t *pAddress)
{
   TAL_RESULT      Error         = TAL_ERROR;
   static uint8_t bMACRetrieved  = TAL_FALSE;
   static uint8_t  MACAddress[6] = {0x00,0x11,0x22,0x33,0x44,0x55};
   uint8_t         Buffer[6];
   int             ReadCnt;
   
   if (TAL_FALSE == bMACRetrieved)
   {
      bMACRetrieved = TAL_TRUE;

      /* Sets up board specific I2C interface */
      Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 27, IOCON_FUNC1);
      Chip_IOCON_PinMuxSet(LPC_IOCON, 0, 28, IOCON_FUNC1);
   
      /* Initialize I2C */
      Chip_I2C_Init(I2C0);
      Chip_I2C_SetClockRate(I2C0, 100000);

      /* Set I2C mode to polling */
      Chip_I2C_SetMasterEventHandler(I2C0, Chip_I2C_EventHandlerPolling);
      
      /* Get MAC address from the onboard EEprom (0xA0>>1 = 0x50) at offset 0xFA */
      ReadCnt = Chip_I2C_MasterCmdRead(I2C0, 0x50, 0xFA, Buffer, 6);
      if (6 == ReadCnt)
      {
         memcpy(MACAddress, Buffer, 6);
         Error = TAL_OK;
      }
      
      Chip_I2C_DeInit(I2C0);
   }
   
   /* Return MAC address */
   memcpy(pAddress, MACAddress, 6);
   
   return(Error);
} /* tal_BoardGetMACAddress */

#endif /* USE_BOARD_BF4088 */

/*** EOF ***/
