/**************************************************************************
*  Copyright (c) 2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  18.04.2018  mifi  First Version.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdio.h>
#include "tal.h"
#include "terminal.h"

#include "ipstack.h"
#include "ipweb.h"
#include "iperf.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/* 
 * Some TASK variables like stack and task control block.
 */
static OS_STACK (StartStack,  TASK_START_STK_SIZE);
static OS_STACK (LEDStack,    TASK_LED_STK_SIZE);

static os_tcb_t TCBStartTask;
static os_tcb_t TCBLed;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  OutputBootMessage                                                    */
/*                                                                       */
/*  Output boot message.                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputBootMessage (void)
{
   const char ResetScreen[] = { 0x1B, 'c', 0 };
   
   term_printf("%s", ResetScreen);
   OSTimeDly(50);   
   
   term_printf("\n");
   term_printf("*********************************\n");
   term_printf("  Project: %s\n", PROJECT_NAME);
   term_printf("  Board  : %s\n", TAL_BOARD);
   term_printf("  Version: v%s\n", PROJECT_VER_STRING);
   term_printf("  Build  : "__DATE__ " " __TIME__"\n");
   term_printf("*********************************\n");
   term_printf("\n");
} /* OutputBootMessage */

/*************************************************************************/
/*  OutputVersionInfo                                                    */
/*                                                                       */
/*  Output version information.                                          */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputVersionInfo (void)
{
   term_printf("*** Version ***\n");
   term_printf("OS   : %s v%s\n", OS_NAME, OS_VER_STRING);
   term_printf("TAL  : v%s\n", TAL_CORE_VER_STRING);
   term_printf("Board: %s\n", TAL_BOARD);
   term_printf("CPU  : %s\n", TAL_CPU);
   term_printf("\n");
} /* OutputVersionInfo */

/*************************************************************************/
/*  OutputFrequencyInfo                                                  */
/*                                                                       */
/*  Output frequency information.                                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputFrequencyInfo (void)
{
   term_printf("*** Frequency info ***\r\n");
   term_printf("CPU: %3d MHz\r\n", tal_CPUGetFrequencyCPU() / 1000000); 
   term_printf("APB: %3d MHz\r\n", tal_CPUGetFrequencyAPB() / 1000000); 
   term_printf("EMC: %3d MHz\r\n", tal_CPUGetFrequencyEMC() / 1000000); 
   term_printf("USB: %3d MHz\r\n", tal_CPUGetFrequencyUSB() / 1000000); 
   term_printf("OSC: %3d MHz\r\n", OSCILLATOR_CLOCK_FREQUENCY / 1000000); 
   term_printf("\r\n");
} /* OutputFrequencyInfo */

/*************************************************************************/
/*  OutputCPULoad                                                        */
/*                                                                       */
/*  Output load information.                                             */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OutputCPULoad (void)
{
   term_printf("CPU load: %d%%\n\n", OSStatGetCPULoad());
} /* OutputCPULoad */

/*************************************************************************/
/*  OutputUsageInfo                                                      */
/*                                                                       */
/*  Output usage information.                                            */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputUsageInfo (void)
{
   term_printf("*** Usage ***\n");
   term_printf("h: output this info\n");
   term_printf("?: output this info\n");
   term_printf("c: output cpu load info\n");
   term_printf("f: output frequency info\n");
   term_printf("r: output runtime stack info\n");
   term_printf("t: output task info\n");
   term_printf("v: output version info\n");
   term_printf("\n");
} /* OutputUsageInfo */

/*************************************************************************/
/*  LEDTask                                                              */
/*                                                                       */
/*  This is the LED task.                                                */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void LEDTask (void *p)
{
   (void)p;
   
   while (1)
   {
      tal_LEDSet(LED1);
      OSTimeDly(TASK_LED_DELAY_MS);
      tal_LEDClear(LED1);
      OSTimeDly(TASK_LED_DELAY_MS);
   }
} /* LEDTask */

/*************************************************************************/
/*  StartTask                                                            */
/*                                                                       */
/*  This is the Start task.                                              */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void StartTask (void *p)
{
   (void)p;
   
   /*
    * The StartTask will be used to start all other tasks in the system.
    * At the end the priority will be set to "IDLE" priority.
    */
    
   /*
    * Start the System ticker
    */
   OSSysTickStart();    

   /* 
    * Enable the statistic function 
    */
   OSStatEnable();
   
   /*
    * Start the Terminal functionality 
    */
   term_Start();
   
   /*
    * Output startup messages
    */
   OutputBootMessage();
   OutputUsageInfo();
   
   /* Create the LED task */
   OSTaskCreate(&TCBLed, LEDTask, NULL, TASK_LED_PRIORITY,
                LEDStack, sizeof(LEDStack), 
                "LEDTask");

   /* Start the IP stack */
   IPIfaceStart();

   /* Create the IPerf task */
   iperf_Start();

   /* Start the web server */
   IPWebsStart();

   OSTaskChangePriority(TASK_START_PRIORITY_IDLE);

   while (1)
   {
      tal_LEDSet(LED2);
      OSTimeDly(TASK_START_DELAY_MS);
      tal_LEDClear(LED2);
      OSTimeDly(TASK_START_DELAY_MS);
   }
} /* StartTask */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
int main (void)
{
   /* 
    * Init the Tiny Abstraction Layer 
    */
   tal_Init();
      
   /* 
    * Create the StartTask.
    * The StartTask is the one and only task which 
    * will start all other tasks of the system.
    */
   OSTaskCreate(&TCBStartTask, StartTask, NULL, TASK_START_PRIORITY,
                StartStack, sizeof(StartStack), 
                "StartTask");
   
   /* 
    * OSStart must be the last function here.
    *  
    * Fasten your seatbelt, engine will be started...
    */        
   OSStart();
   
   /*
    * This return here make no sense.
    * But to prevent the compiler warning:
    *    "return type of 'main' is not 'int'
    * We use an int as return :-)
    */ 
   return(0); /*lint !e527*/
} /* main */

/*************************************************************************/
/*  term_RxCallback                                                      */
/*                                                                       */
/*  Will be called from TermTask in case a char is received.             */
/*                                                                       */
/*  In    : bData                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void term_RxCallback (uint8_t bData)
{
   switch (bData)
   {
      case 'h':
      case '?':
      {
         OutputUsageInfo();
         break;
      } 
           
      case 'c':
      {
         OutputCPULoad();
         break;
      }

      case 'f':
      {
         OutputFrequencyInfo();
         break;
      }

      case 'r':
      {
         OSOutputRuntimeStackInfo();
         break;
      }
      
      case 't':
      {
         OSOutputTaskInfo();
         break;
      }
      
      case 'v':
      {
         OutputVersionInfo();
         break;
      }
      
      /****************************************************/
      
      default:
      {
         /* Do nothing */
         break;
      }
   } /* end switch (bData) */
} /* term_RxCallback */

/*** EOF ***/
